<?php

namespace Drupal\monarch_linkit\Plugin\Linkit\Matcher;

use Drupal\Core\Form\FormStateInterface;
use Drupal\linkit\Plugin\Linkit\Matcher\EntityMatcher;
use Drupal\file\Entity\File;

/**
 * Media URL matcher for media entities.
 *
 * Matches the default media matcher (EntityMatcher), but returns the
 * source URL of the media entity instead of the URL for the entity
 * itself.
 *
 * @Matcher(
 *   id = "entity:media_url",
 *   target_entity = "media",
 *   label = @Translation("Media URL"),
 *   provider = "media"
 * )
 */
class MediaSourceMatcher extends EntityMatcher {

  /**
   * {@inheritdoc}
   */
  public function getSummary() {
    $summery = parent::getSummary();

    $summery[] = $this->t('Include inactive: @include_inactive', [
      '@include_inactive' => $this->configuration['include_inactive'] ? $this->t('Yes') : $this->t('No'),
    ]);

    return $summery;
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return parent::defaultConfiguration() + [
        'include_inactive' => FALSE,
      ];
  }

  /**
   * {@inheritdoc}
   */
  public function calculateDependencies() {
    return parent::calculateDependencies() + [
        'module' => ['media'],
      ];
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $form = parent::buildConfigurationForm($form, $form_state);

    $form['include_inactive'] = [
      '#title' => t('Include inactive media'),
      '#type' => 'checkbox',
      '#default_value' => $this->configuration['include_inactive'],
      '#description' => t('In order to see inactive media, the requesting user must also have permissions to do so.'),
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state) {
    parent::submitConfigurationForm($form, $form_state);
    $this->configuration['include_inactive'] = $form_state->getValue('include_inactive');
  }

  /**
   * {@inheritdoc}
   */
  protected function buildEntityQuery($match) {
    $query = parent::buildEntityQuery($match);

    if ($this->configuration['include_inactive'] !== TRUE) {
      $query->condition('status', 1);
    }

    return $query;
  }

  /**
   * {@inheritdoc}
   */
  protected function buildPath($entity) {
    // This plugin targets media entities, so validation not really needed.
    $source = $entity->getSource();

    if (in_array($source->getPluginId(), ['image', 'file'], TRUE)) {
      /*
       * Both of these media sources use file id under the hood. It may be
       * worth converting this to a db_query to avoid loading the entire
       * file entity and add flexibility in grabbing those urls.
       */
      $source_value = $source->getSourceFieldValue($entity);
      $file_entity = File::load($source_value);
      return file_url_transform_relative(file_create_url($file_entity->getFileUri()));
    }

    return $entity->toUrl()->toString();
  }

}
